@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """Main Controller"""
 from tg import TGController
-from tg import expose, flash, require, url, lurl, request, redirect, validate, tmpl_context, config
+from tg import expose, flash, require, url, lurl, request, redirect, validate, tmpl_context, config, abort
 from tg.decorators import before_render, cached_property, paginate
 from tg.i18n import ugettext as _, lazy_ugettext as l_
 
@@ -137,17 +137,23 @@ class RootController(TGController):
 
     @require(predicates.in_group('smallpress'))
     @expose('genshi:smallpress.templates.edit')
-    def edit(self, uid, *args, **kw):
-        inject_datagrid_resources(attachments_table)
+    def edit(self, uid=None, *args, **kw):
+        if not uid:
+            abort(404)
 
         article = DBSession.query(Article).get(uid)
+        if not article:
+            abort(404)
+
+        inject_datagrid_resources(attachments_table)
+
         value = {
-            'uid':article.uid,
-            'title':article.title,
-            'description':article.description,
-            'tags':comma_separated_tags(article),
-            'publish_date':format_date(article),
-            'content':article.content
+            'uid': article.uid,
+            'title': article.title,
+            'description': article.description,
+            'tags': comma_separated_tags(article),
+            'publish_date': format_date(article),
+            'content': article.content
         }
 
         hooks = config['hooks'].get('smallpress.before_edit_article', [])
@@ -192,6 +198,11 @@ class RootController(TGController):
         attachment = Attachment(name=kw['name'], article=article,
                                 content=AttachmentType(kw['file'].file, kw['file'].filename))
         DBSession.add(attachment)
+
+        hooks = config['hooks'].get('smallpress.on_attachment_upload', [])
+        for func in hooks:
+            func(article, attachment)
+
         DBSession.flush()
 
         return dict(value=dict(article=article.uid),
@@ -207,7 +218,13 @@ class RootController(TGController):
     @expose()
     def rmattachment(self, attachment):
         article = attachment.article
+
+        hooks = config['hooks'].get('smallpress.on_attachment_remove', [])
+        for func in hooks:
+            func(article, attachment)
+
         DBSession.delete(attachment)
+
         flash(_('Attachment successfully removed'))
         return redirect(self.mount_point+'/edit/%s'%article.uid)
 
@@ -220,6 +237,11 @@ class RootController(TGController):
         else:
             article.published=True
             flash(_('Article published'))
+
+        hooks = config['hooks'].get('smallpress.on_article_publishing', [])
+        for func in hooks:
+            func(article, True)
+
         return redirect(self.mount_point+'/manage/'+article.blog_name)
 
     @require(predicates.in_group('smallpress'))
@@ -228,12 +250,21 @@ class RootController(TGController):
     def hide(self, article):
         article.published=False
         flash(_('Article hidden'))
+
+        hooks = config['hooks'].get('smallpress.on_article_publishing', [])
+        for func in hooks:
+            func(article, False)
+
         return redirect(self.mount_point+'/manage/'+article.blog_name)
 
     @require(predicates.in_group('smallpress'))
     @validate(dict(article=SQLAEntityConverter(Article)), error_handler=manage)
     @expose()
     def delete(self, article):
+        hooks = config['hooks'].get('smallpress.on_article_delete', [])
+        for func in hooks:
+            func(article)
+
         DBSession.delete(article)
         flash(_('Article successfully removed'))
         return redirect(self.mount_point+'/manage/'+article.blog_name)
